<?php

namespace Drp\SimpleCms;

use InvalidArgumentException;

/**
 * Class ContentFactory
 * @package Drp\SimpleCms
 */
class ContentFactory implements \Drp\SimpleCms\Contracts\ContentFactory
{
    /**
     * Return the latest Content using the type and ID.
     * First get the relevant Model using the type
     * and then get the latest using the ID.
     *
     * @param string $contentType
     * @param integer $contentId
     * @return \Drp\SimpleCms\Content
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException If a matching content was not found.
     */
    public function latestVersionByTypeAndContentId($contentType, $contentId)
    {
        $contentClass = $this->makeContentClass($contentType);

        return $contentClass->getLatestVersionById($contentId);
    }

    /**
     * Return the Content using the model id and the type.
     *
     * @param string $contentType
     * @param integer $versionId
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException If no matching version is found.
     * @return \Drp\SimpleCms\Content
     */
    public static function byTypeAndVersionId($contentType, $versionId)
    {
        return self::makeContentClass($contentType)->findOrFail($versionId);
    }

    /**
     * Return an instance of the content class based on the given content name.
     *
     * @param string $contentType The type of content object to return
     * @return \Drp\SimpleCms\Content
     */
    public static function makeContentClass($contentType)
    {
        return app(self::getContentClassName($contentType));
    }

    /**
     * Return the content class based on the given content name.
     *
     * @param string $contentType The type of content object to return
     * @return string
     */
    private static function getContentClassName($contentType)
    {
        if (!array_key_exists($contentType, config('simple_cms.content_types'))) {
            throw new InvalidArgumentException(trans('content.errors.name-invalid'));
        }

        return array_get(config('simple_cms.content_types'), $contentType);
    }

    /**
     * Return the studies label based on type.
     *
     * @param string $contentType
     * @param boolean $plural
     * @param boolean $lowerCase
     * @return string
     */
    public static function contentLabel($contentType, $plural = null, $lowerCase = null)
    {
        $content = self::makeContentClass($contentType);

        return $content->contentLabel($plural, $lowerCase);
    }
}
